@@ -1,10 +1,37 @@
 from ceph_deploy.util import pkg_managers, templates
-from ceph_deploy.lib.remoto import process
+from ceph_deploy.lib import remoto
+
+
+def repository_url_part(distro):
+    """
+    Historically everything CentOS, RHEL, and Scientific has been mapped to
+    `el6` urls, but as we are adding repositories for `rhel`, the URLs should
+    map correctly to, say, `rhel6` or `rhel7`.
+
+    This function looks into the `distro` object and determines the right url
+    part for the given distro, falling back to `el6` when all else fails.
+
+    Specifically to work around the issue of CentOS vs RHEL::
+
+        >>> platform.linux_distribution()
+        ('Red Hat Enterprise Linux Server', '7.0', 'Maipo')
+
+    """
+    if distro.normalized_name == 'redhat':
+        if distro.release.startswith('6'):
+            return 'rhel6'
+        elif distro.release.startswith('7'):
+            return 'rhel7'
+    return 'el6'
 
 
 def install(distro, version_kind, version, adjust_repos):
+    logger = distro.conn.logger
     release = distro.release
     machine = distro.machine_type
+    repo_part = repository_url_part(distro)
+
+    pkg_managers.yum_clean(distro.conn)
 
     # Even before EPEL, make sure we have `wget`
     pkg_managers.yum(distro.conn, 'wget')
@@ -12,13 +39,14 @@ def install(distro, version_kind, version, adjust_repos):
     # Get EPEL installed before we continue:
     if adjust_repos:
         install_epel(distro)
+        install_yum_priorities(distro)
     if version_kind in ['stable', 'testing']:
         key = 'release'
     else:
         key = 'autobuild'
 
     if adjust_repos:
-        process.run(
+        remoto.process.run(
             distro.conn,
             [
                 'rpm',
@@ -28,11 +56,12 @@ def install(distro, version_kind, version, adjust_repos):
         )
 
         if version_kind == 'stable':
-            url = 'http://ceph.com/rpm-{version}/el6/'.format(
+            url = 'http://ceph.com/rpm-{version}/{repo}/'.format(
                 version=version,
+                repo=repo_part,
                 )
         elif version_kind == 'testing':
-            url = 'http://ceph.com/rpm-testing/el6/'
+            url = 'http://ceph.com/rpm-testing/{repo}/'.format(repo=repo_part)
         elif version_kind == 'dev':
             url = 'http://gitbuilder.ceph.com/ceph-rpm-centos{release}-{machine}-basic/ref/{version}/'.format(
                 release=release.split(".",1)[0],
@@ -40,7 +69,7 @@ def install(distro, version_kind, version, adjust_repos):
                 version=version,
                 )
 
-        process.run(
+        remoto.process.run(
             distro.conn,
             [
                 'rpm',
@@ -50,12 +79,16 @@ def install(distro, version_kind, version, adjust_repos):
             ],
         )
 
-    process.run(
+        # set the right priority
+        logger.warning('ensuring that /etc/yum.repos.d/ceph.repo contains a high pririty')
+        distro.conn.remote_module.set_repo_priority(['Ceph', 'Ceph-noarch', 'ceph-source'])
+        logger.warning('altered ceph.repo priorities to contain: priority=1')
+
+    remoto.process.run(
         distro.conn,
         [
             'yum',
             '-y',
-            '-q',
             'install',
             'ceph',
         ],
@@ -70,7 +103,7 @@ def install_epel(distro):
     if distro.name.lower() in ['centos', 'scientific']:
         distro.conn.logger.info('adding EPEL repository')
         if float(distro.release) >= 6:
-            process.run(
+            remoto.process.run(
                 distro.conn,
                 ['wget', 'http://dl.fedoraproject.org/pub/epel/6/x86_64/epel-release-6-8.noarch.rpm'],
             )
@@ -82,7 +115,7 @@ def install_epel(distro):
                 ],
             )
         else:
-            process.run(
+            remoto.process.run(
                 distro.conn,
                 ['wget', 'http://dl.fedoraproject.org/pub/epel/5/x86_64/epel-release-5-4.noarch.rpm'],
             )
@@ -95,16 +128,47 @@ def install_epel(distro):
             )
 
 
+def install_yum_priorities(distro):
+    """
+    EPEL started packaging Ceph so we need to make sure that the ceph.repo we
+    install has a higher priority than the EPEL repo so that when installing
+    Ceph it will come from the repo file we create.
+
+    The name of the package changed back and forth (!) since CentOS 4:
+
+    From the CentOS wiki::
+
+        Note: This plugin has carried at least two differing names over time.
+        It is named yum-priorities on CentOS-5 but was named
+        yum-plugin-priorities on CentOS-4. CentOS-6 has reverted to
+        yum-plugin-priorities.
+
+    """
+    if distro.normalized_name == 'centos':
+        if distro.release[0] == '6':
+            package_name = 'yum-plugin-priorities'
+        else:
+            package_name = 'yum-priorities'
+
+        pkg_managers.yum(
+            distro.conn,
+            package_name,
+        )
+
+
 def mirror_install(distro, repo_url, gpg_url, adjust_repos):
     repo_url = repo_url.strip('/')  # Remove trailing slashes
+    gpg_url_path = gpg_url.split('file://')[-1]  # Remove file if present
+
+    pkg_managers.yum_clean(distro.conn)
 
     if adjust_repos:
-        process.run(
+        remoto.process.run(
             distro.conn,
             [
                 'rpm',
                 '--import',
-                gpg_url,
+                gpg_url_path,
             ]
         )
 
@@ -121,37 +185,42 @@ def mirror_install(distro, repo_url, gpg_url, adjust_repos):
     pkg_managers.yum(distro.conn, 'ceph')
 
 
-def repo_install(distro, repo_name, baseurl, gpgkey, **kw):
+def repo_install(distro, reponame, baseurl, gpgkey, **kw):
     # Get some defaults
-    name = kw.get('name', '%s repo' % repo_name)
+    name = kw.get('name', '%s repo' % reponame)
     enabled = kw.get('enabled', 1)
     gpgcheck = kw.get('gpgcheck', 1)
     install_ceph = kw.pop('install_ceph', False)
+    proxy = kw.get('proxy')
     _type = 'repo-md'
     baseurl = baseurl.strip('/')  # Remove trailing slashes
 
-    process.run(
-        distro.conn,
-        [
-            'rpm',
-            '--import',
-            gpgkey,
-        ]
-    )
+    pkg_managers.yum_clean(distro.conn)
 
-    repo_content = templates.custom_repo.format(
-        repo_name=repo_name,
+    if gpgkey:
+        remoto.process.run(
+            distro.conn,
+            [
+                'rpm',
+                '--import',
+                gpgkey,
+            ]
+        )
+
+    repo_content = templates.custom_repo(
+        reponame=reponame,
         name = name,
         baseurl = baseurl,
         enabled = enabled,
         gpgcheck = gpgcheck,
         _type = _type,
         gpgkey = gpgkey,
+        proxy = proxy,
     )
 
     distro.conn.remote_module.write_yum_repo(
         repo_content,
-        "%s.repo" % repo_name
+        "%s.repo" % reponame
     )
 
     # Some custom repos do not need to install ceph
